@@ -7,10 +7,11 @@ pytest.register_assert_rewrite("rewrite.test")
 from .execution import ExecutionContext, DelegatingExecutionContext, InMemoryExecutionContext, Recipe, RecipeRunException
 from .markers import *
 from .tree import Checksum, FileAttributes, SourceFile, Tree, PrintOutputCapture, PrinterFactory
-from .utils import random_id
+from .utils import random_id, list_find, list_map, list_map_last
 from .visitor import Cursor, TreeVisitor
 from .parser import *
 from .result import *
+from .style import *
 
 __all__ = [
     'Checksum',
@@ -20,13 +21,22 @@ __all__ = [
     'PrintOutputCapture',
     'PrinterFactory',
     'random_id',
+    'list_find',
+    'list_map',
+    'list_map_last',
     'Cursor',
     'TreeVisitor',
     'ExecutionContext',
     'DelegatingExecutionContext',
     'InMemoryExecutionContext',
+
+    # Markers
     'Marker',
     'Markers',
+    'SearchResult',
+    'UnknownJavaMarker',
+
+    # Execution
     'Recipe',
     'RecipeRunException',
     'Parser',
@@ -36,5 +46,10 @@ __all__ = [
     'ParserBuilder',
     'ParseExceptionResult',
     'Result',
-    'SourceFile'
-]
\ No newline at end of file
+    'SourceFile',
+
+    # Style
+    'Style',
+    'NamedStyles',
+    'GeneralFormatStyle',
+]
