@@ -1,28 +1,33 @@
+from dataclasses import replace
 from typing import Optional, TypeVar, TYPE_CHECKING
 
 from .support_types import J, JRightPadded, JLeftPadded, JContainer, Space
+from rewrite.utils import list_map
+from rewrite.visitor import Cursor
+from rewrite.tree import Tree
 
 if TYPE_CHECKING:
     from .visitor import JavaVisitor
     from .tree import *
 
+P = TypeVar('P')
 T = TypeVar('T')
 J2 = TypeVar('J2', bound=J)
 
 
-def visit_container(v: 'JavaVisitor', container: Optional[JContainer[J2]], loc: JContainer.Location, p) -> Optional[JContainer[J2]]:
+def visit_container(v: 'JavaVisitor', container: Optional[JContainer[J2]], loc: JContainer.Location, p: P) -> Optional[JContainer[J2]]:
     if container is None:
         return None
 
     v.cursor = Cursor(v.cursor, container)
     before = v.visit_space(container.before, loc.before_location, p)
-    js = [v.visit_right_padded(el.element, loc.element_location, p) for el in container.padding.elements]
+    js = list_map(lambda el: v.visit_right_padded(el, loc.element_location, p), container.padding.elements)
     v.cursor = v.cursor.parent
 
-    return container if js == container.padding.elements and before is container.before else JContainer(before, js, container.markers)
+    return container if js is container.padding.elements and before is container.before else JContainer(before, js, container.markers)
 
 
-def visit_right_padded(v: 'JavaVisitor', right: Optional[JRightPadded[T]], loc: JRightPadded.Location, p) -> Optional[JRightPadded[T]]:
+def visit_right_padded(v: 'JavaVisitor', right: Optional[JRightPadded[T]], loc: JRightPadded.Location, p: P) -> Optional[JRightPadded[T]]:
     if right is None:
         return None
 
@@ -41,7 +46,7 @@ def visit_right_padded(v: 'JavaVisitor', right: Optional[JRightPadded[T]], loc:
     return right
 
 
-def visit_left_padded(v: 'JavaVisitor', left: Optional[JLeftPadded[T]], loc: JLeftPadded.Location, p) -> Optional[JLeftPadded[T]]:
+def visit_left_padded(v: 'JavaVisitor', left: Optional[JLeftPadded[T]], loc: JLeftPadded.Location, p: P) -> Optional[JLeftPadded[T]]:
     if left is None:
         return None
 
@@ -60,7 +65,7 @@ def visit_left_padded(v: 'JavaVisitor', left: Optional[JLeftPadded[T]], loc: JLe
     return JLeftPadded(before, t, left.markers)
 
 
-def visit_space(v: 'JavaVisitor', space: Optional[Space], loc: Space.Location, p):
+def visit_space(v: 'JavaVisitor', space: Optional[Space], loc: Space.Location, p: P) -> Space:
     # FIXME support Javadoc
     return space
 
