@@ -4,6 +4,9 @@ from .support_types import *
 from .parser import *
 from .visitor import *
 from .markers import *
+from .style import *
+from .format import *
+from .templating import *
 
 __all__ = [
     'Py',
@@ -23,8 +26,10 @@ __all__ = [
     # AST types
     'Async',
     'Await',
+    'Binary',
     'ChainedAssignment',
     'CollectionLiteral',
+    'CompilationUnit',
     'ComprehensionExpression',
     'Del',
     'DictLiteral',
@@ -40,6 +45,7 @@ __all__ = [
     'MultiImport',
     'NamedArgument',
     'Pass',
+    'PyComment',
     'Slice',
     'SpecialParameter',
     'Star',
@@ -51,4 +57,24 @@ __all__ = [
     'UnionType',
     'VariableScope',
     'YieldFrom',
+
+    # Style
+    'PythonStyle',
+    'SpacesStyle',
+    'TabsAndIndentsStyle',
+    'WrappingAndBracesStyle',
+    'BlankLinesStyle',
+    'OtherStyle',
+    'IntelliJ',
+
+    # Formatter
+    'AutoFormat',
+    'BlankLinesVisitor',
+    'MinimumViableSpacingVisitor',
+    'NormalizeFormatVisitor',
+    'NormalizeTabsOrSpacesVisitor',
+    'SpacesVisitor',
+
+    # Templating
+    'PythonTemplate',
 ]
