@@ -4,7 +4,7 @@ from operator import attrgetter
 from typing import Optional, TypeVar, cast
 
 from rewrite import Tree, TreeVisitor, Markers, Marker, Cursor
-from rewrite.remote import Sender, SenderContext
+from rewrite_remote import Sender, SenderContext
 from rewrite.python import *
 from rewrite.java import *
 from rewrite.python.support_types import *
@@ -426,9 +426,10 @@ class PythonSender(Sender):
             ctx.send_node(case, attrgetter('_prefix'), PythonSender.send_space)
             ctx.send_node(case, attrgetter('_markers'), ctx.send_markers)
             ctx.send_value(case, attrgetter('_type'))
-            ctx.send_node(case, attrgetter('_expressions'), PythonSender.send_container)
+            ctx.send_node(case, attrgetter('_case_labels'), PythonSender.send_container)
             ctx.send_node(case, attrgetter('_statements'), PythonSender.send_container)
             ctx.send_node(case, attrgetter('_body'), PythonSender.send_right_padded)
+            ctx.send_node(case, attrgetter('_guard'), ctx.send_tree)
             return case
 
         def visit_class_declaration(self, class_declaration: ClassDeclaration, ctx: SenderContext) -> J:
@@ -589,6 +590,15 @@ class PythonSender(Sender):
             ctx.send_typed_value(instance_of, attrgetter('_type'))
             return instance_of
 
+        def visit_deconstruction_pattern(self, deconstruction_pattern: DeconstructionPattern, ctx: SenderContext) -> J:
+            ctx.send_value(deconstruction_pattern, attrgetter('_id'))
+            ctx.send_node(deconstruction_pattern, attrgetter('_prefix'), PythonSender.send_space)
+            ctx.send_node(deconstruction_pattern, attrgetter('_markers'), ctx.send_markers)
+            ctx.send_node(deconstruction_pattern, attrgetter('_deconstructor'), ctx.send_tree)
+            ctx.send_node(deconstruction_pattern, attrgetter('_nested'), PythonSender.send_container)
+            ctx.send_typed_value(deconstruction_pattern, attrgetter('_type'))
+            return deconstruction_pattern
+
         def visit_intersection_type(self, intersection_type: IntersectionType, ctx: SenderContext) -> J:
             ctx.send_value(intersection_type, attrgetter('_id'))
             ctx.send_node(intersection_type, attrgetter('_prefix'), PythonSender.send_space)
@@ -787,6 +797,7 @@ class PythonSender(Sender):
             ctx.send_node(switch_expression, attrgetter('_markers'), ctx.send_markers)
             ctx.send_node(switch_expression, attrgetter('_selector'), ctx.send_tree)
             ctx.send_node(switch_expression, attrgetter('_cases'), ctx.send_tree)
+            ctx.send_typed_value(switch_expression, attrgetter('_type'))
             return switch_expression
 
         def visit_synchronized(self, synchronized: Synchronized, ctx: SenderContext) -> J:
@@ -935,6 +946,13 @@ class PythonSender(Sender):
             ctx.send_value(source, attrgetter('_text'))
             return source
 
+        def visit_erroneous(self, erroneous: Erroneous, ctx: SenderContext) -> J:
+            ctx.send_value(erroneous, attrgetter('_id'))
+            ctx.send_node(erroneous, attrgetter('_prefix'), PythonSender.send_space)
+            ctx.send_node(erroneous, attrgetter('_markers'), ctx.send_markers)
+            ctx.send_value(erroneous, attrgetter('_text'))
+            return erroneous
+
     @classmethod
     def send_container(cls, container: JContainer[T], ctx: SenderContext):
         extensions.send_container(container, ctx)
